from flask import Flask, request, jsonify
import sqlite3
from typing import Dict, Any, Optional, List
import csv


class NiceServer:
    """Класс для упрощенного создания Flask-сервера с поддержкой базы данных."""

    def __init__(self, port: int = 5000, host: str = "0.0.0.0", debug: bool = False):
        self.app = Flask(__name__)
        self.host = host
        self.port = port
        self.debug = debug
        self.routes = {}  # Словарь для хранения функций маршрутов
        self.db_path = None  # Путь к базе данных
        self.table_name = None  # Имя таблицы
        self.columns = None  # Список колонок для базы данных

    def register_function(self, func):
        """Регистрирует функцию как маршрут, используя ее имя как URL."""
        url = f"/{func.__name__}"

        def wrapped():
            try:
                if request.method == "GET":
                    args = request.args.to_dict()
                else:
                    args = request.get_json() or {}
                result = func(url, **args)
                return jsonify({"result": result, "status": "success"})
            except Exception as e:
                return jsonify({"error": str(e), "status": "error"}), 400

        self.routes[url] = func
        self.app.route(url, methods=["GET", "POST"])(wrapped)
        return func

    def Login(self, db_path: str = "users.db", table_name: str = "users", **columns):
        """Инициализирует базу данных для логина и регистрации."""
        self.db_path = db_path
        self.table_name = table_name
        self.columns = columns

        column_defs = []
        for col_name, col_type in columns.items():
            if col_type == int:
                column_defs.append(f"{col_name} INTEGER DEFAULT 0")
            elif col_type == float:
                column_defs.append(f"{col_name} REAL DEFAULT 0.0")
            elif col_type == str:
                column_defs.append(f"{col_name} TEXT DEFAULT NULL")
            else:
                raise ValueError(f"Unsupported column type for {col_name}: {col_type}")

        conn = sqlite3.connect(self.db_path)
        cursor = conn.cursor()
        create_table_query = f"CREATE TABLE IF NOT EXISTS {self.table_name} ({', '.join(column_defs)})"
        cursor.execute(create_table_query)
        conn.commit()
        conn.close()

    def register(self, **kwargs) -> bool:
        """Регистрирует пользователя в базе данных."""
        if not self.db_path or not self.table_name or not self.columns:
            raise ValueError("Database not initialized. Call Login() first.")

        data = {}
        for col_name, col_type in self.columns.items():
            if col_name in kwargs:
                data[col_name] = kwargs[col_name]
            else:
                if col_type == int or col_type == float:
                    data[col_name] = 0
                elif col_type == str:
                    data[col_name] = None

        columns = ', '.join(data.keys())
        placeholders = ', '.join(['?' for _ in data])
        values = tuple(data.values())

        conn = sqlite3.connect(self.db_path)
        cursor = conn.cursor()
        insert_query = f"INSERT INTO {self.table_name} ({columns}) VALUES ({placeholders})"
        try:
            cursor.execute(insert_query, values)
            conn.commit()
            conn.close()
            return True
        except sqlite3.IntegrityError:
            conn.close()
            return False

    def get_info(self, **kwargs) -> Optional[Dict[str, Any]]:
        """Ищет пользователя по двум любым параметрам, возвращает все данные или None."""
        if not self.db_path or not self.table_name or not self.columns:
            raise ValueError("Database not initialized. Call Login() first.")

        if len(kwargs) != 2:
            raise ValueError("Exactly two parameters are required for search.")

        conditions = [f"{key} = ?" for key in kwargs.keys()]
        where_clause = " AND ".join(conditions)
        values = tuple(kwargs.values())

        conn = sqlite3.connect(self.db_path)
        cursor = conn.cursor()
        select_query = f"SELECT * FROM {self.table_name} WHERE {where_clause}"
        cursor.execute(select_query, values)
        result = cursor.fetchone()
        conn.close()

        if result:
            return dict(zip(self.columns.keys(), result))
        return None

    def update_user(self, identifier: str, id_value: Any, **kwargs) -> bool:
        """Обновляет данные пользователя по заданному идентификатору (например, id)."""
        if not self.db_path or not self.table_name or not self.columns:
            raise ValueError("Database not initialized. Call Login() first.")

        if not kwargs:
            raise ValueError("At least one field to update is required.")

        set_clause = ', '.join([f"{key} = ?" for key in kwargs.keys()])
        values = tuple(kwargs.values()) + (id_value,)

        conn = sqlite3.connect(self.db_path)
        cursor = conn.cursor()
        update_query = f"UPDATE {self.table_name} SET {set_clause} WHERE {identifier} = ?"
        try:
            cursor.execute(update_query, values)
            conn.commit()
            affected = cursor.rowcount
            conn.close()
            return affected > 0
        except Exception as e:
            conn.close()
            raise e

    def delete_user(self, **kwargs) -> bool:
        """Удаляет пользователя по двум параметрам."""
        if not self.db_path or not self.table_name or not self.columns:
            raise ValueError("Database not initialized. Call Login() first.")

        if len(kwargs) != 2:
            raise ValueError("Exactly two parameters are required for deletion.")

        conditions = [f"{key} = ?" for key in kwargs.keys()]
        where_clause = " AND ".join(conditions)
        values = tuple(kwargs.values())

        conn = sqlite3.connect(self.db_path)
        cursor = conn.cursor()
        delete_query = f"DELETE FROM {self.table_name} WHERE {where_clause}"
        try:
            cursor.execute(delete_query, values)
            conn.commit()
            affected = cursor.rowcount
            conn.close()
            return affected > 0
        except Exception as e:
            conn.close()
            raise e

    def count_users(self) -> int:
        """Возвращает общее количество пользователей в таблице."""
        if not self.db_path or not self.table_name or not self.columns:
            raise ValueError("Database not initialized. Call Login() first.")

        conn = sqlite3.connect(self.db_path)
        cursor = conn.cursor()
        cursor.execute(f"SELECT COUNT(*) FROM {self.table_name}")
        count = cursor.fetchone()[0]
        conn.close()
        return count

    def add_column(self, name: str, type: type) -> None:
        """Добавляет новую колонку в таблицу."""
        if not self.db_path or not self.table_name or not self.columns:
            raise ValueError("Database not initialized. Call Login() first.")

        if name in self.columns:
            raise ValueError(f"Column {name} already exists.")

        if type == int:
            col_def = f"{name} INTEGER DEFAULT 0"
        elif type == float:
            col_def = f"{name} REAL DEFAULT 0.0"
        elif type == str:
            col_def = f"{name} TEXT DEFAULT NULL"
        else:
            raise ValueError(f"Unsupported column type for {name}: {type}")

        conn = sqlite3.connect(self.db_path)
        cursor = conn.cursor()
        alter_query = f"ALTER TABLE {self.table_name} ADD COLUMN {col_def}"
        try:
            cursor.execute(alter_query)
            conn.commit()
            self.columns[name] = type
            conn.close()
        except Exception as e:
            conn.close()
            raise e

    def drop_column(self, name: str) -> None:
        """Удаляет колонку из таблицы."""
        if not self.db_path or not self.table_name or not self.columns:
            raise ValueError("Database not initialized. Call Login() first.")

        if name not in self.columns:
            raise ValueError(f"Column {name} does not exist.")

        conn = sqlite3.connect(self.db_path)
        cursor = conn.cursor()
        try:
            cursor.execute(f"PRAGMA table_info({self.table_name})")
            cols = [col[1] for col in cursor.fetchall() if col[1] != name]
            new_cols = ', '.join(cols)
            cursor.execute(f"CREATE TABLE temp_table AS SELECT {new_cols} FROM {self.table_name}")
            cursor.execute(f"DROP TABLE {self.table_name}")
            cursor.execute(f"ALTER TABLE temp_table RENAME TO {self.table_name}")
            conn.commit()
            del self.columns[name]
            conn.close()
        except Exception as e:
            conn.close()
            raise e

    def clear_table(self) -> None:
        """Очищает таблицу, удаляя все записи."""
        if not self.db_path or not self.table_name or not self.columns:
            raise ValueError("Database not initialized. Call Login() first.")

        conn = sqlite3.connect(self.db_path)
        cursor = conn.cursor()
        try:
            cursor.execute(f"DELETE FROM {self.table_name}")
            conn.commit()
            conn.close()
        except Exception as e:
            conn.close()
            raise e

    def hash_password(self, password: str) -> str:
        """Хеширует пароль для безопасного хранения."""
        if not self.db_path or not self.table_name or not self.columns:
            raise ValueError("Database not initialized. Call Login() first.")

        try:
            import bcrypt
            salt = bcrypt.gensalt()
            hashed = bcrypt.hashpw(password.encode('utf-8'), salt)
            return hashed.decode('utf-8')
        except ImportError:
            raise ImportError("bcrypt not installed. Install it using 'pip install bcrypt'")

    def export_to_csv(self, path: str) -> None:
        """Экспортирует таблицу в CSV-файл."""
        if not self.db_path or not self.table_name or not self.columns:
            raise ValueError("Database not initialized. Call Login() first.")

        conn = sqlite3.connect(self.db_path)
        cursor = conn.cursor()
        cursor.execute(f"SELECT * FROM {self.table_name}")
        rows = cursor.fetchall()
        headers = list(self.columns.keys())

        with open(path, 'w', newline='', encoding='utf-8') as f:
            writer = csv.writer(f)
            writer.writerow(headers)
            writer.writerows(rows)

        conn.close()

    def run(self):
        """Запускает сервер."""
        self.app.run(host=self.host, port=self.port, debug=self.debug)